﻿using Syncfusion.DataSource;
using Syncfusion.ListView.XForms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection.Emit;
using System.Text;

using Xamarin.Forms;

namespace ListViewSample
{
    public class MainPage : ContentPage
    {
        #region Fields

        ContactsViewModel viewModel;
        SfListView listView;

        #endregion

        #region Constructor

        public MainPage()
        {
            Content = InitializeView();
            Title = "SfListView Grouping Sample";
        }

        #endregion

        #region Private Methods

        private Grid InitializeView()
        {
            var grid = new Grid();
            var rowdefinition = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            grid.RowDefinitions.Add(rowdefinition);
            InitializeListView();
            grid.Children.Add(listView);

            return grid;
        }

        private void InitializeListView()
        {
            listView = new SfListView();
            viewModel = new ContactsViewModel();
            listView.ItemsSource = viewModel.contactsinfo;
            listView.ItemTemplate = new DataTemplate(() => { return CreateItemTemplate(); });
            listView.GroupHeaderTemplate = new DataTemplate(() => { return CreateGroupHeaderTemplate(); });
            listView.ItemSize = 70;
            listView.GroupHeaderSize = 60;
            listView.AllowGroupExpandCollapse = true;
            listView.ItemSpacing = new Thickness(0, 0, 5, 0);
            listView.IsStickyHeader = true;
            listView.DataSource.SortDescriptors.Add(new SortDescriptor()
            {
                PropertyName = "ContactName",
                Direction = ListSortDirection.Ascending
            });

            listView.DataSource.GroupDescriptors.Add(new GroupDescriptor()
            {
                PropertyName = "ContactName",
                KeySelector = (object obj1) =>
                {
                    var item = (obj1 as Contacts);
                    return item.ContactName[0].ToString();
                },
            });
            listView.Loaded += ListView_Loaded;
        }

        private void ListView_Loaded(object sender, ListViewLoadedEventArgs e)
        {
            listView.CollapseAll();
        }

        private Grid CreateItemTemplate()
        {
            var gridView = new Grid();
            gridView.RowSpacing = 1;
            var row0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var row1 = new RowDefinition { Height = 1 };
            gridView.RowDefinitions.Add(row0);
            gridView.RowDefinitions.Add(row1);

            var grid = new Grid();

            var column0 = new ColumnDefinition { Width = 50 };
            var column1 = new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) };
            var column2 = new ColumnDefinition { Width = 70 };
            grid.ColumnDefinitions.Add(column0);
            grid.ColumnDefinitions.Add(column1);
            grid.ColumnDefinitions.Add(column2);

            var contactimage = new Image();
            contactimage.SetBinding(Image.SourceProperty, new Binding("ContactImage"));
            contactimage.HeightRequest = 50;
            contactimage.VerticalOptions = LayoutOptions.Center;
            contactimage.HorizontalOptions = LayoutOptions.Center;

            var gridview = new Grid();

            gridview.RowSpacing = 1;
            gridview.Padding = new Thickness(10, 0, 0, 0);
            gridview.VerticalOptions = LayoutOptions.Center;
            var rowdefinition0 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            var rowdefinition1 = new RowDefinition { Height = new GridLength(1, GridUnitType.Star) };
            gridview.RowDefinitions.Add(rowdefinition0);
            gridview.RowDefinitions.Add(rowdefinition1);

            var contactName = new Label();
            contactName.SetBinding(Label.TextProperty, new Binding("ContactName"));
            contactName.LineBreakMode = LineBreakMode.NoWrap;
            contactName.FontAttributes = FontAttributes.Bold;
            contactName.TextColor = Color.FromHex("#474747");
            contactName.FontSize = 20;

            var contactNumber = new Label();
            contactNumber.FontAttributes = FontAttributes.None;
            contactNumber.FontSize = 18;
            contactNumber.LineBreakMode = LineBreakMode.NoWrap;
            contactNumber.SetBinding(Label.TextProperty, new Binding("ContactNumber"));
            contactNumber.TextColor = Color.FromHex("#474747");

            var contactType = new Label();
            contactType.Margin = new Thickness(5);
            contactType.FontAttributes = FontAttributes.None;
            contactType.LineBreakMode = LineBreakMode.NoWrap;
            contactType.VerticalOptions = LayoutOptions.End;
            contactType.VerticalTextAlignment = TextAlignment.End;
            contactType.HorizontalOptions = LayoutOptions.End;
            contactType.SetBinding(Label.TextProperty, new Binding("ContactType"));
            contactType.TextColor = Color.FromHex("#474747");
            contactType.FontSize = 12;

            var stackLayout = new StackLayout();
            stackLayout.HeightRequest = 1;
            stackLayout.BackgroundColor = Color.Gray;

            gridview.Children.Add(contactName, 0, 0);
            gridview.Children.Add(contactNumber, 0, 1);
            gridview.Children.Add(contactType, 1, 0);

            grid.Children.Add(contactimage);
            grid.Children.Add(gridview, 1, 0);

            gridView.Children.Add(grid);
            gridView.Children.Add(stackLayout, 0, 1);

            return gridView;
        }

        private Grid CreateGroupHeaderTemplate()
        {
            var grid = new Grid()
            {
                BackgroundColor = Color.FromHex("#E4E4E4")
            };
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            grid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
            var grouping = new StackLayout()
            {
                VerticalOptions = LayoutOptions.Center,
                HorizontalOptions = LayoutOptions.Start,
                Orientation = StackOrientation.Horizontal,
                Padding = new Thickness(0, 0, 0, 0)
            };
            var yearLabel = new Label
            {
                TextColor = Color.FromHex("#474747"),
                FontSize = 22,
                FontAttributes = FontAttributes.Bold
            };
            var groupingImg = new Image();
            //{
            //    Source = ImageSource.FromResource("BaseKit.Resources.arrow_down_white.png"),
            //};
            groupingImg.SetBinding(Image.SourceProperty, "IsExpand", BindingMode.Default, new BoolToImageConverter(), null);
            yearLabel.SetBinding(Label.TextProperty, new Binding("Key"));
            grouping.Children.Add(groupingImg);
            grouping.Children.Add(yearLabel);

            var counting = new StackLayout()
            {
                VerticalOptions = LayoutOptions.Center,
                HorizontalOptions = LayoutOptions.EndAndExpand,
                Orientation = StackOrientation.Horizontal
            };
            var countLabel = new Label
            {
                TextColor = Color.FromHex("#474747"),
                FontSize = 22
            };
            countLabel.SetBinding(Label.TextProperty, new Binding("Count", BindingMode.Default, null, null, "{0,3:###}"));
            var countlabel2 = new Label() { Text = "Trips:", TextColor = Color.FromHex("#474747"), FontSize =22};
            counting.Children.Add(countlabel2);
            counting.Children.Add(countLabel);
            grid.Children.Add(grouping);
            grid.Children.Add(counting, 1, 0);
            return grid;
        }

        #endregion
    }
}
